
/* qemu address layout:
 * 0-xxx ram
 *   0x01000000 kernel
 *   0x017ff000 cmdline
 *   0x01800000 initrd
 *
 *   <platform dependent>
 *  isa, pci io & memory
 *
 *   0xfff80000 rom
 *   0xfffffffc entry point / initial pc
 */

/* qemu nvram:
 * 0x00 16 signature	QEMU_BIOS
 * 0x10  4 version	22
 * 0x14  1 size		x256, 0x0400 - 0x2000
 * 0x20 16 arch		PREP
 * 0x30  4 memsize
 * 0x34  1 boot device	m, a, b, c, d
 * 0x38  4 kernel_addr	0x01000000
 * 0x3c  4 kernel_len
 * 0x40  4 cmdline_addr	0x017ff000
 * 0x44  4 cmdline_len
 * 0x48  4 initrd_addr	0x01800000 when loaded
 * 0x4c  4 initrd_len
 * 0x50  4 nvram_image	to write to nvram?
 * 0x54  2 vga width
 * 0x55  2 vga height
 * 0x58  2 vga depth
 * 0xFC  4 crc
 *
 */

/* reg usage:
 * r31 base
 * r30 = nvram in ram
 * r3 = dt in ram
 */

top = 0x01000000
nvram_copy = top - 256

nvram_l	= 0x74
nvram_h	= 0x75
nvram_d	= 0x77

#define r0 0
#define r1 1
#define r2 2
#define r3 3
#define r4 4
#define r5 5
#define r6 6
#define r7 7
#define r30 30
#define r31 31

#include "qemu-dt.S"

	.balign 4
start:
	mflr	r31	/* base */
	lis	r30, nvram_copy@ha
	addi	r30, r30, nvram_copy@l
	
	/* copy 256 bytes of nvram to ram */
	lwz	r6, dt_pci_io_base - base(r31)
	li	r4, 256
	mtctr	r4
	stb	r4, nvram_h(r6)
1:
	addi	r4, r4, -1
	stb	r4, nvram_l(r6)
	lbz	r5, nvram_d(r6)
	stbx	r5, r30, r4
	bdnz	1b

	/* santiy checks */

	/* copy dtb to just before nvram */
	addi	r6, r31, _dt_blob_start - base
	lwz	r4, 4(r6)	/* totalsize */
	addi	r4, r4, 3
	rlwinm	r4, r4, 0, 0, 29
	subf	r3, r4, r30

1:
	addi	r4, r4,-4
	cmpwi	r4, 0
	lwzx	r5, r6, r4
	stwx	r5, r3, r4
	bne	1b

	/* patch dtb: memsize, initrd and command line */
	lwz	r5, 0x30(r30)	/* memory size */
	stw	r5, dt_mem_size - _dt_blob_start(r3)
	lwz	r5, 0x48(r30)	/* initrd start */
	stw	r5, dt_initrd_start - _dt_blob_start(r3)
	lwz	r6, 0x4c(r30)	/* initrd len */
	add	r6, r6, r5
	stw	r6, dt_initrd_end - _dt_blob_start(r3)

	addi	r6, r3, dt_cmdline - _dt_blob_start
	lwz	r4, 0x44(r30)	/* cmdline len */
	lwz	r5, -8(r6)	/* prop_length */
	cmplw	r4, r5
	blt	1f
	addi	r4, r5, -1
1:	
	cmpwi	r4, 0
	mtctr	r4
	lwz	r4, 0x40(r30)	/* cmdline start */
	beq	2f
	addi	r4, r4, -1
	addi	r6, r6, -1
1:
	lbzu	r5, 1(r4)
	stbu	r5, 1(r6)
	bdnz	1b
2:

	/* find kernel entry point and go to it */
	lwz	r6, 0x38(r30)	/* kernel start (elf header)  */
	lhz	r4, 44(r6)	/* prog header count */
	cmpwi	r4, 0
	mtctr	r4
	lwz	r7, 28(r6)	/* prog header offset */
	add	r7, r6, r7
	beq	3f

1:
	lwz	r5, 0(r7)	/* ph type */
	cmpwi	r5, 1		/* PT_LOAD */
	bne	2f

	lwz	r5, 4(r7)	/* ph offset */
	add	r4, r6, r5
	lwz	r5, 8(r7)	/* ph vaddr */
	subf	r4, r5, r4
	lwz	r5, 24(r6)	/* e_entry */
	add	r4, r4, r5
	mtctr	r4
	addi	r1,r3,-256	/* !!! head_32 calls C wtihout setting stack */
	li	r5, 0
	bctrl
	b	$		/* unexpected return from kernel */

2:
	lhz	r4, 42(r6)	/* prog header size */
	add	r7, r7, r4
	bdnz	1b

3:
	b	$		/* no load program header */

	.balign	8		/* dt has align 8 , bl must be last */
	nop
	bl	start
base:
